@@ -52,8 +52,6 @@ UPSCOMMANDS ups_event[] = {
    {"commok",        0},           /* CMDCOMMOK */
    {"startselftest", 0},           /* CMDSTARTSELFTEST */
    {"endselftest",   0},           /* CMDENDSELFTEST */
-   {"mastertimeout", 0},           /* CMDMASTERTIMEOUT */
-   {"masterconnect", 0},           /* CMDMASTERCONN */
    {"offbattery",    0},           /* CMDOFFBATTERY */
    {"battdetach",    0},           /* CMDBATTDETACH */
    {"battattach",    0}            /* CMDBATTATTACH */
@@ -76,13 +74,11 @@ UPSCMDMSG event_msg[] = {
    {LOG_ALERT,   N_("Users requested to logoff.")},
    {LOG_ALERT,   N_("Battery failure. Emergency.")},
    {LOG_CRIT,    N_("UPS battery must be replaced.")},
-   {LOG_CRIT,    N_("Remote shutdown requested")},
+   {LOG_CRIT,    N_("Remote shutdown requested.")},
    {LOG_WARNING, N_("Communications with UPS lost.")},
    {LOG_WARNING, N_("Communications with UPS restored.")},
    {LOG_ALERT,   N_("UPS Self Test switch to battery.")},
    {LOG_ALERT,   N_("UPS Self Test completed.")},
-   {LOG_CRIT,    N_("Master not responding.")},
-   {LOG_WARNING, N_("Connect from master.")},
    {LOG_CRIT,    N_("Mains returned. No longer on UPS batteries.")},
    {LOG_CRIT,    N_("Battery disconnected.")},
    {LOG_CRIT,    N_("Battery reattached.")}
@@ -133,8 +129,6 @@ void generate_event(UPSINFO *ups, int event)
    case CMDPOWEROUT:
    case CMDONBATTERY:
    case CMDOFFBATTERY:
-   case CMDMASTERTIMEOUT:
-   case CMDMASTERCONN:
    case CMDBATTDETACH:
    case CMDBATTATTACH:
    default:
@@ -150,7 +144,7 @@ void generate_event(UPSINFO *ups, int event)
  * ok = 2  => power failure
  * ok = 3  => remote shutdown
  */
-void powerfail(int ok)
+static void powerfail(int ok)
 {
    /*
     * If apcupsd terminates here, it will never get a chance to
@@ -187,14 +181,14 @@ void powerfail(int ok)
  * If called with zero, prevent users from logging in. 
  * If called with one, allow users to login.
  */
-void logonfail(int ok)
+static void logonfail(UPSINFO *ups, int ok)
 {
    int lgnfd;
 
-   unlink(NOLOGIN);
+   unlink(ups->nologinpath);
 
    if (ok == 0 &&
-       ((lgnfd = open(NOLOGIN, O_CREAT | O_WRONLY, 0644)) >= 0)) {
+       ((lgnfd = open(ups->nologinpath, O_CREAT | O_WRONLY, 0644)) >= 0)) {
       write(lgnfd, POWERFAIL, strlen(POWERFAIL));
       close(lgnfd);
    }
@@ -205,9 +199,8 @@ static void prohibit_logins(UPSINFO *ups)
    if (ups->nologin_file)
       return;                      /* already done */
 
-   make_file(ups, NOLOGIN);
+   logonfail(ups, 0);
    ups->nologin_file = true;
-   logonfail(0);
 
    log_event(ups, LOG_ALERT, _("User logins prohibited"));
 }
@@ -221,7 +214,7 @@ static void do_shutdown(UPSINFO *ups, int cmdtype)
    ups->set_shutdown();
    delete_lockfile(ups);
    ups->set_fastpoll();
-   make_file(ups, PWRFAIL);
+   make_file(ups, ups->pwrfailpath);
    prohibit_logins(ups);
 
    if (!ups->is_slave()) {
@@ -404,10 +397,13 @@ void do_action(UPSINFO *ups)
       }
    }
 
-   /* Must SHUTDOWN Remote System Calls */
+   /* Remote is shutting down, so must we. */
    if (ups->is_shut_remote()) {
-      ups->clear_onbatt_msg();
-      generate_event(ups, CMDREMOTEDOWN);
+      if (ups->chg_shut_remote()) {
+         generate_event(ups, CMDREMOTEDOWN);
+      }
+      ups->PrevStatus = ups->Status;
+      write_unlock(ups);
       return;
    }
 
@@ -451,8 +447,8 @@ void do_action(UPSINFO *ups)
       break;
 
    case st_SelfTest:
-      /* allow 20 seconds max for selftest */
-      if (now - ups->SelfTest < 20 && !ups->is_battlow())
+      /* allow 40 seconds max for selftest */
+      if (now - ups->SelfTest < 40 && !ups->is_battlow())
          break;
 
       /* Cancel self test, announce power failure */
@@ -484,16 +480,18 @@ void do_action(UPSINFO *ups)
       } else {                     /* not shutdown yet */
          /*
           * Did BattLow bit go high? Then the battery power is failing.
-          * Normal Power down during Power Failure
+          * Normal Power down during Power Failure: Start shutdown timer.
           */
          if (ups->chg_battlow() && ups->is_battlow()) {
-            generate_event(ups, CMDFAILING);
-            break;
+            Dmsg0(100, "BATTLOW shutdown\n");
+            ups->start_shut_lbatt = now;
          }
+         if (ups->chg_battlow() && !ups->is_battlow())
+            Dmsg0(100, "BATTLOW glitch\n");
 
          /*
           * Did MaxTimeOnBattery Expire?  (TIMEOUT in apcupsd.conf)
-          * Normal Power down during Power Failure
+          * Normal Power down during Power Failure: Shutdown immediately.
           */
          if ((ups->maxtime > 0) && ((now - ups->last_time_on_line) > ups->maxtime)) {
             ups->set_shut_btime();
@@ -503,14 +501,44 @@ void do_action(UPSINFO *ups)
 
          /*
           * Did Battery Charge or Runtime go below percent cutoff?
-          * Normal Power down during Power Failure
+          * Normal Power down during Power Failure: Start shutdown timer.
           */
          if (ups->UPS_Cap[CI_BATTLEV] && ups->BattChg <= ups->percent) {
-            ups->set_shut_load();
+            if (!ups->is_shut_load()) {
+               Dmsg0(100, "CI_BATTLEV shutdown\n");
+               ups->set_shut_load();
+               ups->start_shut_load = now;
+            }
+         } else {
+            if (ups->UPS_Cap[CI_BATTLEV] && ups->is_shut_load())
+               Dmsg0(100, "CI_BATTLEV glitch\n");
+            ups->clear_shut_load();
+         }
+
+         if (ups->UPS_Cap[CI_RUNTIM] && ups->TimeLeft <= ups->runtime) {
+            if (!ups->is_shut_ltime()) {
+               Dmsg0(100, "CI_RUNTIM shutdown\n");
+               ups->set_shut_ltime();
+               ups->start_shut_ltime = now;
+            }
+         } else {
+            if (ups->UPS_Cap[CI_RUNTIM] && ups->is_shut_ltime())
+               Dmsg0(100, "CI_RUNTIM glitch\n");
+            ups->clear_shut_ltime();
+         }
+
+         /*
+          * Check for expired shutdown timers and act on them.
+          */
+         if (ups->is_battlow() && ((now - ups->start_shut_lbatt) >= 5)) {
+            generate_event(ups, CMDFAILING);
+            break;
+         }
+         if (ups->is_shut_load() && ((now - ups->start_shut_load) >= 5)) {
             generate_event(ups, CMDLOADLIMIT);
             break;
-         } else if (ups->UPS_Cap[CI_RUNTIM] && ups->TimeLeft <= ups->runtime) {
-            ups->set_shut_ltime();
+         }
+         if (ups->is_shut_ltime() && ((now - ups->start_shut_ltime) >= 5)) {
             generate_event(ups, CMDRUNLIMIT);
             break;
          }
@@ -521,11 +549,8 @@ void do_action(UPSINFO *ups)
           *
           * Then the battery has failed!!!
           * Must do Emergency Shutdown NOW
-          *
-          * Or of the UPS says he is going to shutdown, do it NOW!
           */
-         if (ups->is_shutdownimm() ||
-            (ups->is_battlow() && ups->is_online())) {
+         if (ups->is_battlow() && ups->is_online()) {
             ups->set_shut_emerg();
             generate_event(ups, CMDEMERGENCY);
          }
@@ -578,15 +603,6 @@ void do_action(UPSINFO *ups)
          generate_event(ups, CMDOFFBATTERY);
       }
 
-      if (ups->is_shutdown()) {
-         /* If we have a shutdown to cancel, do it now. */
-         ups->ShutDown = 0;
-         ups->clear_shutdown();
-         powerfail(1);
-         unlink(PWRFAIL);
-         log_event(ups, LOG_ALERT, _("Cancelling shutdown"));
-      }
-
       if (ups->SelfTest) {
          ups->LastSelfTest = ups->SelfTest;
          ups->SelfTest = 0;
@@ -603,7 +619,7 @@ void do_action(UPSINFO *ups)
       if (ups->nologin_file)
          log_event(ups, LOG_ALERT, _("Allowing logins"));
 
-      logonfail(1);
+      logonfail(ups, 1);
       ups->nologin_file = false;
       requested_logoff = false;
       device_entry_point(ups, DEVICE_CMD_DTR_ST_DISABLE, NULL);
