@@ -85,6 +85,7 @@ static int usb_get_battery_date(void);
 static void usb_set_battery_date(void);
 static void usb_get_manf_date(void);
 static void usb_set_alarm(void);
+static void usb_set_sens(void);
 #endif
 
 static void strip_trailing_junk(char *cmd);
@@ -205,7 +206,6 @@ void apctest_error_out(const char *file, int line, const char *fmt, ...)
    avsnprintf((char *)&buf[i], sizeof(buf) - i, (char *)fmt, arg_ptr);
    va_end(arg_ptr);
 
-   fprintf(stderr, "%s", buf);
    pmsg(buf);
 
    apctest_error_cleanup(core_ups);     /* finish the work */
@@ -233,9 +233,6 @@ void apctest_error_exit(const char *fmt, ...)
 /* Main program */
 
 /* This application must be linked as console app. */
-#ifdef HAVE_CYGWIN
-#undef main
-#endif
 
 #define M_SMART  1
 #define M_DUMB   2
@@ -291,42 +288,6 @@ int main(int argc, char *argv[])
       pmsg("sharenet.type = SHARE\n");
       break;
 
-   case NET:
-      pmsg("sharenet.type = NET\n");
-
-      switch (ups->upsclass.type) {
-      case NO_CLASS:
-         pmsg("upsclass.type = NO_CLASS\n");
-         break;
-      case STANDALONE:
-         pmsg("upsclass.type = STANDALONE\n");
-         break;
-      case SHARESLAVE:
-         pmsg("upsclass.type = SHARESLAVE\n");
-         break;
-      case SHAREMASTER:
-         pmsg("upsclass.type = SHAREMASTER\n");
-         break;
-      case SHARENETMASTER:
-         pmsg("upsclass.type = SHARENETMASTER\n");
-         break;
-      case NETSLAVE:
-         pmsg("upsclass.type = NETSLAVE\n");
-         break;
-      case NETMASTER:
-         pmsg("upsclass.type = NETMASTER\n");
-         break;
-      default:
-         pmsg("upsclass.type = DEFAULT\n");
-         break;
-      }
-      break;
-
-   case SHARENET:
-      pmsg("sharenet.type = SHARENET\n");
-      pmsg("I cannot handle sharenet.type = SHARENET\n");
-      apctest_terminate(1);
-
    default:
       pmsg("sharenet.type = DEFAULT\n");
       pmsg("I cannot handle sharenet.type = DEFAULT\n");
@@ -440,10 +401,6 @@ int main(int argc, char *argv[])
       pmsg("mode.type = SHAREBASIC\n");
       break;
 
-   case NETUPS:
-      pmsg("mode.type = NETUPS\n");
-      break;
-
    case BKPRO:
       pmsg("mode.type = BKPRO\n");
       break;
@@ -468,6 +425,10 @@ int main(int argc, char *argv[])
       pmsg("mode.type = SHARESMART\n");
       break;
 
+   case APCSMART_UPS:
+      pmsg("mode.type = APCSMART_UPS\n");
+      break;
+
    case USB_UPS:
       pmsg("mode.type = USB_UPS\n");
       break;
@@ -490,46 +451,8 @@ int main(int argc, char *argv[])
 
    delete_lockfile(ups);
 
-   switch (ups->sharenet.type) {
-   case DISABLE:
-   case SHARE:
-      pmsg("Setting up the port ...\n");
-      setup_device(ups);
-      break;
-
-   case NET:
-      switch (ups->upsclass.type) {
-      case NO_CLASS:
-      case STANDALONE:
-      case SHARESLAVE:
-      case SHAREMASTER:
-      case SHARENETMASTER:
-         break;
-      case NETSLAVE:
-         if (kill_ups_power)
-            Error_abort0(_("Ignoring killpower for slave\n"));
-         if (prepare_slave(ups))
-            Error_abort0(_("Error setting up slave\n"));
-         break;
-      case NETMASTER:
-         setup_device(ups);
-         if ((kill_ups_power == 0) && (prepare_master(ups)))
-            Error_abort0("Error setting up master\n");
-         break;
-      default:
-         Error_abort1(_("NET Class Error %s\n\a"), strerror(errno));
-      }
-      break;
-
-   case SHARENET:
-      setup_device(ups);
-      if ((kill_ups_power == 0) && (prepare_master(ups)))
-         Error_abort0("Error setting up master.\n");
-      break;
-
-   default:
-      Error_abort0(_("Unknown share net type\n"));
-   }
+   pmsg("Setting up the port ...\n");
+   setup_device(ups);
 
    if (kill_ups_power) {
       pmsg("apctest: bad option, I cannot do a killpower\n");
@@ -546,13 +469,13 @@ int main(int argc, char *argv[])
          prep_device(ups);
       }
 
-#ifdef HAVE_APCSMART_DRIVER
       /*
        * This isn't a documented option but can be used
        * for testing dumb mode on a SmartUPS if you have
        * the proper cable.
        */
       if (dumb_mode_test) {
+#ifdef HAVE_APCSMART_DRIVER
          char ans[20];
 
          write(ups->fd, "R", 1);   /* enter dumb mode */
@@ -560,8 +483,10 @@ int main(int argc, char *argv[])
          getline(ans, sizeof(ans), ups);
          pmsg("Going dumb: %s\n", ans);
          mode = M_DUMB;            /* run in dumb mode */
-      }
+#else
+         pmsg("apcsmart not compiled: dumb mode test unavailable\n");
 #endif
+      }
    }
 
    shm_OK = 1;
@@ -874,9 +799,9 @@ static void test6(void)
 
    ptime();
    print_bits(bits);
-   make_file(ups, PWRFAIL);
+   make_file(ups, ups->pwrfailpath);
    kill_power(ups);
-   unlink(PWRFAIL);
+   unlink(ups->pwrfailpath);
    ptime();
 
    pmsg("returned from kill_power function.\n");
@@ -1393,11 +1318,11 @@ static void program_smart_eeprom(void)
             break;
 
          default:
-            pmsg("Illegal response. Please enter 1-7\n");
+            pmsg("Illegal response. Please enter 1-15\n");
             break;
          }
       } else {
-         pmsg("Illegal response. Please enter 1-7\n");
+         pmsg("Illegal response. Please enter 1-15\n");
       }
    }
    ptime();
@@ -1625,7 +1550,8 @@ static void do_usb_testing(void)
            "5) View battery date\n"
            "6) View manufacturing date\n"
            "7) Set alarm behavior\n"
-           "8) Quit\n\n");
+           "8) Set sensitivity\n"
+           "9) Quit\n\n");
 
       cmd = get_cmd("Select function number: ");
       if (cmd) {
@@ -1654,14 +1580,17 @@ static void do_usb_testing(void)
             usb_set_alarm();
             break;
          case 8:
+            usb_set_sens();
+            break;
+         case 9:
             quit = TRUE;
             break;
          default:
-            pmsg("Illegal response. Please enter 1-7\n");
+            pmsg("Illegal response. Please enter 1-9\n");
             break;
          }
       } else {
-         pmsg("Illegal response. Please enter 1-7\n");
+         pmsg("Illegal response. Please enter 1-9\n");
       }
    }
    ptime();
@@ -1673,6 +1602,87 @@ static void do_usb_testing(void)
 
 #ifdef HAVE_USB_DRIVER
 
+static void usb_set_sens(void)
+{
+   int result;
+   char* cmd;
+
+   if (!usb_read_int_from_ups(ups, CI_SENS, &result)) {
+      pmsg("\nI don't know how to control the alarm settings on your UPS.\n");
+      return;
+   }
+   
+   pmsg("Current sensitivity setting: ");
+   switch(result) {
+   case 0:
+      pmsg("LOW\n");
+      break;
+   case 1:
+      pmsg("MEDIUM\n");
+      break;
+   case 2:
+      pmsg("HIGH\n");
+      break;
+   default:
+      pmsg("UNKNOWN\n");
+      break;
+   }
+
+   while(1) {
+      pmsg("Press...\n"
+           " L for Low sensitivity\n"
+           " M for Medium sensitivity\n"
+           " H for High sensitivity\n"
+           " Q to Quit with no changes\n"
+           "Your choice: ");
+      cmd = get_cmd("Select function: ");
+      if (cmd) {
+         switch(tolower(*cmd)) {
+         case 'l':
+            usb_write_int_to_ups(ups, CI_SENS, 0, "CI_SENS");
+            break;
+         case 'm':
+            usb_write_int_to_ups(ups, CI_SENS, 1, "CI_SENS");
+            break;
+         case 'h':
+            usb_write_int_to_ups(ups, CI_SENS, 2, "CI_SENS");
+            break;
+         case 'q':
+            return;
+         default:
+            pmsg("Illegal response.\n");
+            continue;
+         }
+      } else {
+         pmsg("Illegal response.\n");
+         continue;
+      }
+      
+      break;
+   }
+   
+   /* Delay needed for readback to work */
+   sleep(1);
+
+   usb_read_int_from_ups(ups, CI_SENS, &result);
+   pmsg("New sensitivity setting: ");
+   switch(result) {
+   case 0:
+      pmsg("LOW\n");
+      break;
+   case 1:
+      pmsg("MEDIUM\n");
+      break;
+   case 2:
+      pmsg("HIGH\n");
+      break;
+   default:
+      pmsg("UNKNOWN\n");
+      break;
+   }
+   
+}
+
 static void usb_set_alarm(void)
 {
    int result;
@@ -1725,6 +1735,9 @@ static void usb_set_alarm(void)
       break;
    }
    
+   /* Delay needed for readback to work */
+   sleep(1);
+
    usb_read_int_from_ups(ups, CI_DALARM, &result);
    pmsg("New alarm setting: ");
    switch(result) {
@@ -1738,7 +1751,6 @@ static void usb_set_alarm(void)
       pmsg("UNKNOWN\n");
       break;
    }
-   
 }
 
 static void usb_kill_power_test(void)
@@ -1755,9 +1767,9 @@ static void usb_kill_power_test(void)
    ptime();
    pmsg("calling kill_power function.\n");
 
-   make_file(ups, PWRFAIL);
+   make_file(ups, ups->pwrfailpath);
    kill_power(ups);
-   unlink(PWRFAIL);
+   unlink(ups->pwrfailpath);
 
    ptime();
    pmsg("returned from kill_power function.\n");
@@ -1848,7 +1860,7 @@ static void usb_run_self_test(void)
 
    pmsg("Waiting for test to complete...");
 
-   for (timeout = 0; timeout < 20; timeout++) {
+   for (timeout = 0; timeout < 40; timeout++) {
       if (!usb_read_int_from_ups(ups, CI_ST_STAT, &result)) {
          pmsg("ERROR READING STATUS\n");
          usb_write_int_to_ups(ups, CI_ST_STAT, 3, "SelftestStatus");
@@ -1863,7 +1875,7 @@ static void usb_run_self_test(void)
       sleep(1);
    }
 
-   if (timeout == 20) {
+   if (timeout == 40) {
       pmsg("TEST DID NOT COMPLETE\n");
       usb_write_int_to_ups(ups, CI_ST_STAT, 3, "SelftestStatus");
       return;
@@ -2100,7 +2112,11 @@ static void parse_eeprom_cmds(char *eprom, char locale)
 {
    char *p = eprom;
    char c, l, n, s;
+#ifdef debuggggggg
+   int i;
+#endif
 
+   ncmd = 0;
    for (;;) {
       c = *p++;
       if (c == 0)
