@@ -161,19 +161,35 @@ int getline(char *s, int len, UPSINFO *ups)
    int ending = 0;
    char c;
    int retval;
+   int wait;
+
+   if (s != NULL)
+      wait = TIMER_FAST;   /* 1 sec, expect fast response */
+   else
+      wait = ups->wait_time;
+
+#ifdef HAVE_MINGW
+   /* Set read() timeout since we have no select() support. */
+   {
+      COMMTIMEOUTS ct;
+      HANDLE h = (HANDLE)_get_osfhandle(ups->fd);
+      ct.ReadIntervalTimeout = MAXDWORD;
+      ct.ReadTotalTimeoutMultiplier = MAXDWORD;
+      ct.ReadTotalTimeoutConstant = wait * 1000;
+      ct.WriteTotalTimeoutMultiplier = 0;
+      ct.WriteTotalTimeoutConstant = 0;
+      SetCommTimeouts(h, &ct);
+   }
+#endif
 
    while (!ending) {
-#ifndef HAVE_CYGWIN
+#if !defined(HAVE_MINGW)
       fd_set rfds;
       struct timeval tv;
 
       FD_ZERO(&rfds);
       FD_SET(ups->fd, &rfds);
-      if (s != NULL) {
-         tv.tv_sec = TIMER_FAST;   /* 1 sec, expect fast response */
-      } else {
-         tv.tv_sec = ups->wait_time;
-      }
+      tv.tv_sec = wait;
       tv.tv_usec = 0;
 
       errno = 0;
@@ -191,7 +207,6 @@ int getline(char *s, int len, UPSINFO *ups)
       default:
          break;
       }
-
 #endif
 
       do {
@@ -731,10 +746,6 @@ int apcsmart_ups_entry_point(UPSINFO *ups, int command, void *data)
    int retries = 5;                /* Number of retries if reason is NA (see below) */
    char ans[20];
 
-   if (ups->is_slave()) {
-      return SUCCESS;
-   }
-
    switch (command) {
    case DEVICE_CMD_SET_DUMB_MODE:
       /* Set dumb mode for a smart UPS */
