@@ -141,13 +141,6 @@ int snmp_ups_open(UPSINFO *ups)
       }
 
       memset(Sid->MIB, 0, sizeof(powernet_mib_t));
-      if (snmp_ups_get_capabilities(ups) == 0) {        /* Check Comm */
-         Error_abort0(_("PANIC! Cannot communicate with UPS via SNMP.\n\n"
-               "Please make sure the port specified on the DEVICE directive\n"
-               "is correct, that your able UPSCABLE is smart and the\n"
-               "remote SNMP UPS is running and reachable.\n"));
-         return 0;
-      }
 
       /* Run powernet specific init */
       return powernet_snmp_ups_open(ups);
@@ -161,13 +154,6 @@ int snmp_ups_open(UPSINFO *ups)
       }
 
       memset(Sid->MIB, 0, sizeof(ups_mib_t));
-      if (snmp_ups_get_capabilities(ups) == 0) {        /* Check Comm */
-         Error_abort0(_("PANIC! Cannot communicate with UPS via SNMP.\n\n"
-               "Please make sure the port specified on the DEVICE directive\n"
-               "is correct, that your able UPSCABLE is smart and the\n"
-               "remote SNMP UPS is running and reachable.\n"));
-         return 0;
-      }
       return 1;
    }
 
