@@ -61,9 +61,12 @@ const struct s_known_info known_info[] = {
    {CI_NONE,                    0x00840033, P_ANY,     T_UNITS,    false},  /* ApparentPower */
    {CI_NONE,                    0x00840034, P_ANY,     T_UNITS,    false},  /* ActivePower */
    {CI_LOAD,                    0x00840035, P_ANY,     T_UNITS,    true },  /* PercentLoad */
-   {CI_ITEMP,                   0x00840036, P_BATTERY, T_UNITS,    true },  /* Temperature */
+   {CI_ITEMP,                   0x00840036, P_BATTERY, T_UNITS,    true },  /* Internal Temperature */
+   {CI_ATEMP,                   0x00840036, P_APC1,    T_UNITS,    true },  /* Ambient Temperature */
    {CI_HUMID,                   0x00840037, P_ANY,     T_UNITS,    true },  /* Humidity */
-   {CI_NOMBATTV,                0x00840040, P_ANY,     T_UNITS,    false},  /* ConfigVoltage */
+   {CI_NOMBATTV,                0x00840040, P_BATTERY, T_UNITS,    false},  /* ConfigVoltage (battery) */
+   {CI_NOMOUTV,                 0x00840040, P_OUTPUT,  T_UNITS,    false},  /* ConfigVoltage (output) */
+   {CI_NOMINV,                  0x00840040, P_INPUT,   T_UNITS,    false},  /* ConfigVoltage (input) */
    {CI_NONE,                    0x00840042, P_ANY,     T_UNITS,    false},  /* ConfigFrequency */
    {CI_NONE,                    0x00840043, P_ANY,     T_UNITS,    false},  /* ConfigApparentPower */
    {CI_LTRANS,                  0x00840053, P_ANY,     T_UNITS,    false},  /* LowVoltageTransfer */
@@ -137,6 +140,7 @@ const struct s_known_info known_info[] = {
    {CI_APCDelayBeforeStartup,   0xFF86007E, P_ANY,     T_UNITS,    false},  /* APCDelayBeforeStartup */
    {CI_APCDelayBeforeShutdown,  0xFF86007D, P_ANY,     T_UNITS,    false},  /* APCDelayBeforeShutdown */
    {CI_APCLineFailCause,        0xFF860052, P_ANY,     T_NONE,     true},   /* APCLineFailCause */
+   {CI_SENS,                    0xFF860061, P_ANY,     T_NONE,     false},  /* APCSensitivity */
    {CI_BUPBattCapBeforeStartup, 0x00860012, P_ANY,     T_NONE,     false},  /* BUPBattCapBeforeStartup */
    {CI_BUPDelayBeforeStartup,   0x00860076, P_ANY,     T_NONE,     false},  /* BUPDelayBeforeStartup */
    {CI_BUPSelfTest,             0x00860010, P_ANY,     T_NONE,     false},  /* BUPSelfTest */
@@ -250,6 +254,7 @@ static bool usb_get_value(UPSINFO *ups, int ci, USB_VALUE *uval)
 {
    static struct timeval prev = {0};
    struct timeval now;
+   struct timespec delay;
    int diff;
 
    /*
@@ -260,8 +265,11 @@ static bool usb_get_value(UPSINFO *ups, int ci, USB_VALUE *uval)
    if (prev.tv_sec) {
       gettimeofday(&now, NULL);
       diff = TV_DIFF_MS(prev, now);
-      if (diff >= 0 && diff < URB_DELAY_MS)
-          usleep((URB_DELAY_MS-diff)*1000);
+      if (diff >= 0 && diff < URB_DELAY_MS) {
+         delay.tv_sec = 0;
+         delay.tv_nsec = (URB_DELAY_MS-diff)*1000000;
+         nanosleep(&delay, NULL);
+      }
    }
    gettimeofday(&prev, NULL);
 
@@ -419,7 +427,7 @@ static void usb_process_value(UPSINFO* ups, int ci, USB_VALUE* uval)
 
    /*  Ambient temperature */ 
    case CI_ATEMP:
-      ups->ambtemp = uval->dValue;
+      ups->ambtemp = uval->dValue - 273.15;      /* convert to deg C. */;
       Dmsg1(200, "ATemp = %d\n", (int)ups->ambtemp);
       break;
 
@@ -525,14 +533,22 @@ static void usb_process_value(UPSINFO* ups, int ci, USB_VALUE* uval)
    /* model, firmware */
    case CI_UPSMODEL:
       /* Truncate Firmware info on APC Product string */
-      if ((p = strchr(uval->sValue, 'F')) && *(p + 1) == 'W' && *(p + 2) == ':') {
-         *(p - 1) = 0;
-         astrncpy(ups->firmrev, p + 4, sizeof(ups->firmrev));
+      if ((p = strstr(uval->sValue, "FW:"))) {
+         *p = '\0';           // Terminate model name
+         p += 3;              // Skip "FW:"
+         while (isspace(*p))  // Skip whitespace after "FW:"
+            p++;
+         astrncpy(ups->firmrev, p, sizeof(ups->firmrev));
          ups->UPS_Cap[CI_REVNO] = true;
       }
 
-      astrncpy(ups->upsmodel, uval->sValue, sizeof(ups->upsmodel));
-      astrncpy(ups->mode.long_name, uval->sValue, sizeof(ups->mode.long_name));
+      /* Kill leading whitespace on model name */
+      p = uval->sValue;
+      while (isspace(*p))
+         p++;
+
+      astrncpy(ups->upsmodel, p, sizeof(ups->upsmodel));
+      astrncpy(ups->mode.long_name, p, sizeof(ups->mode.long_name));
       break;
 
    /* WAKEUP_DELAY */
@@ -610,11 +626,34 @@ static void usb_process_value(UPSINFO* ups, int ci, USB_VALUE* uval)
       ups->NomOutputVoltage = (int)uval->dValue;
       break;
 
+   /* Nominal input voltage */
+   case CI_NOMINV:
+      ups->NomInputVoltage = (int)uval->dValue;
+      break;
+
    /* Nominal battery voltage */
    case CI_NOMBATTV:
       ups->nombattv = uval->dValue;
       break;
 
+   /* Sensitivity */
+   case CI_SENS:
+      switch (uval->iValue) {
+      case 0:
+         astrncpy(ups->sensitivity, "Low", sizeof(ups->sensitivity));
+         break;
+      case 1:
+         astrncpy(ups->sensitivity, "Medium", sizeof(ups->sensitivity));
+         break;
+      case 2:
+         astrncpy(ups->sensitivity, "High", sizeof(ups->sensitivity));
+         break;
+      default:
+         astrncpy(ups->sensitivity, "Unknown", sizeof(ups->sensitivity));
+         break;
+      }
+      break;
+
    default:
       break;
    }
@@ -635,6 +674,8 @@ static bool usb_update_value(UPSINFO* ups, int ci)
 /* Process commands from the main loop */
 int usb_ups_entry_point(UPSINFO *ups, int command, void *data)
 {
+   struct timespec delay = {0, 40000000};
+
    switch (command) {
    case DEVICE_CMD_CHECK_SELFTEST:
       Dmsg0(80, "Checking self test.\n");
@@ -645,7 +686,7 @@ int usb_ups_entry_point(UPSINFO *ups, int command, void *data)
        * entry point.
        */
       /* Reason for last transfer to batteries */
-      usleep(40000);  /* Give UPS a chance to update the value */
+      nanosleep(&delay, NULL); /* Give UPS a chance to update the value */
       if (usb_update_value(ups, CI_WHY_BATT) ||
           usb_update_value(ups, CI_APCLineFailCause))
       {
@@ -663,6 +704,7 @@ int usb_ups_entry_point(UPSINFO *ups, int command, void *data)
       break;
 
    case DEVICE_CMD_GET_SELFTEST_MSG:
+      nanosleep(&delay, NULL); /* Give UPS a chance to update the value */
       return usb_update_value(ups, CI_ST_STAT);
 
    default:
@@ -1065,8 +1107,8 @@ int usb_report_event(UPSINFO *ups, int ci, USB_VALUE *uval)
    case CI_Discharging:
    case CI_ACPresent:
    case CI_BelowRemCapLimit:
-   case CI_RemainingCapacity:
-   case CI_RunTimeToEmpty:
+   case CI_BATTLEV:
+   case CI_RUNTIM:
    case CI_NeedReplacement:
    case CI_ShutdownImminent:
    case CI_BatteryPresent:
