@@ -13,8 +13,7 @@
  */
 
 /*
- * Copyright (C) 1999 Kern Sibbald
- * Copyright (C) 1996-99 Andre M. Hedrick <andre@suse.com>
+ * Copyright (C) 1999-2006 Kern Sibbald
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General
@@ -45,6 +44,11 @@ char *net_errmsg = NULL;           /* pointer to error message */
 char net_errbuf[256];              /* error message buffer for messages */
 
 
+#ifdef HAVE_MINGW
+#define close(fd)           closesocket(fd)
+#endif
+
+
 /*
  * Read nbytes from the network.
  * It is possible that the total bytes require in several
@@ -53,49 +57,35 @@ char net_errbuf[256];              /* error message buffer for messages */
 
 static int read_nbytes(int fd, char *ptr, int nbytes)
 {
-   int nleft, nread;
-
-#if defined HAVE_OPENBSD_OS || defined HAVE_FREEBSD_OS
+   int nleft, nread = 0;
+   struct timeval timeout;
    int rc;
    fd_set fds;
-#endif
 
    nleft = nbytes;
 
    while (nleft > 0) {
       do {
-#if defined HAVE_OPENBSD_OS || defined HAVE_FREEBSD_OS
-         /* 
-          * Work around a bug in OpenBSD & FreeBSD userspace pthreads
-          * implementations.
-          *
-          * The pthreads implementation under the hood sets O_NONBLOCK
-          * implicitly on all fds. This setting is not visible to the user
-          * application but is relied upon by the pthreads library to prevent
-          * blocking syscalls in one thread from halting all threads in the
-          * process. When a process exit()s or exec()s, the implicit
-          * O_NONBLOCK flags are removed from all fds, EVEN THOSE IT INHERITED.
-          * If another process is still using the inherited fds, there will
-          * soon be trouble.
-          *
-          * apcupsd is bitten by this issue after fork()ing a child process to
-          * run apccontrol.
-          *
-          * select() is conveniently immune to the O_NONBLOCK issue so we use
-          * that to make sure the following read() will not block.
-          */
-         do {
-            FD_ZERO(&fds);
-            FD_SET(fd, &fds);
-            rc = select(fd + 1, &fds, NULL, NULL, NULL);
-         } while (rc == -1 && (errno == EINTR || errno == EAGAIN));
+         /* Expect data from the server within 15 seconds */
+         timeout.tv_sec = 15;
+         timeout.tv_usec = 0;
 
-         if (rc < 0) {
+         FD_ZERO(&fds);
+         FD_SET(fd, &fds);
+
+         rc = select(fd + 1, &fds, NULL, NULL, &timeout);
+
+         switch (rc) {
+         case -1:
+            if (errno == EINTR || errno == EAGAIN)
+               continue;
             net_errno = errno;
-            return (-1);           /* error */
+            return -1;           /* error */
+         case 0:
+            return 0;            /* timeout */
          }
-#endif
-         nread = read(fd, ptr, nleft);
+
+         nread = recv(fd, ptr, nleft, 0);
       } while (nread == -1 && (errno == EINTR || errno == EAGAIN));
 
       if (nread <= 0) {
@@ -121,15 +111,28 @@ static int write_nbytes(int fd, char *ptr, int nbytes)
    nleft = nbytes;
    while (nleft > 0) {
 #if defined HAVE_OPENBSD_OS || defined HAVE_FREEBSD_OS
-      /* 
+      /*       
        * Work around a bug in OpenBSD & FreeBSD userspace pthreads
-       * implementations. Rationale is the same as described above.
+       * implementations.
+       *
+       * The pthreads implementation under the hood sets O_NONBLOCK
+       * implicitly on all fds. This setting is not visible to the user
+       * application but is relied upon by the pthreads library to prevent
+       * blocking syscalls in one thread from halting all threads in the
+       * process. When a process exit()s or exec()s, the implicit
+       * O_NONBLOCK flags are removed from all fds, EVEN THOSE IT INHERITED.
+       * If another process is still using the inherited fds, there will
+       * soon be trouble.
+       *
+       * apcupsd is bitten by this issue after fork()ing a child process to
+       * run apccontrol.
+       *
        * This seemingly-pointless fcntl() call causes the pthreads
        * library to reapply the O_NONBLOCK flag appropriately.
        */
       fcntl(fd, F_SETFL, fcntl(fd, F_GETFL));
 #endif
-      nwritten = write(fd, ptr, nleft);
+      nwritten = send(fd, ptr, nleft, 0);
 
       if (nwritten <= 0) {
          net_errno = errno;
@@ -268,7 +271,7 @@ int net_open(char *host, char *service, int port)
    fcntl(sockfd, F_SETFL, fcntl(sockfd, F_GETFL));
 #endif
 
-   if (connect(sockfd, (struct sockaddr *)&tcp_serv_addr, sizeof(tcp_serv_addr)) < 0) {
+   if (connect(sockfd, (struct sockaddr *)&tcp_serv_addr, sizeof(tcp_serv_addr)) == -1) {
       asnprintf(net_errbuf, sizeof(net_errbuf),
          _("tcp_open: cannot connect to server %s on port %d.\n"
         "ERR=%s\n"), host, port, strerror(errno));
@@ -283,10 +286,6 @@ int net_open(char *host, char *service, int port)
 /* Close the network connection */
 void net_close(int sockfd)
 {
-   short pktsiz = 0;
-
-   /* send EOF sentinel */
-   write_nbytes(sockfd, (char *)&pktsiz, sizeof(short));
    close(sockfd);
 }
 
@@ -297,7 +296,12 @@ void net_close(int sockfd)
  */
 int net_accept(int fd, struct sockaddr_in *cli_addr)
 {
+#ifdef HAVE_MINGW                                       
+   /* kludge because some idiot defines socklen_t as unsigned */
+   int clilen = sizeof(*cli_addr);
+#else
    socklen_t clilen = sizeof(*cli_addr);
+#endif
    int newfd;
 
 #if defined HAVE_OPENBSD_OS || defined HAVE_FREEBSD_OS
