@@ -11,9 +11,10 @@
 #include   <mapi.h>
 #include   <stdio.h>
 
+#ifndef HAVE_MINGW
 extern void mainCRTStartup();
-
 void WinMainCRTStartup() { mainCRTStartup(); }
+#endif
 
 int main(int argc, char **argv)
 {
