@@ -25,7 +25,7 @@
 // by Kern E. Sibbald.  Many thanks to ATT and James Weatherall,
 // the original author, for providing an excellent template.
 //
-// Copyright (2000-2005) Kern E. Sibbald
+// Copyright (2000-2006) Kern E. Sibbald
 //     20 July 2000
 //
 
@@ -36,16 +36,19 @@
 
 // winmain.cpp for win32 version of apcupsd
 
+#include "apc.h"
+#include "winapi.h"
+#undef STRICT
+
 ////////////////////////////
 // System headers
 #include <unistd.h>
-#include "winhdrs.h"
+#include <windows.h>
 #include <lmcons.h>
 #include <ctype.h>
 #include <signal.h>
 #include <pthread.h>
 #include <errno.h>
-#include "config.h"
 #include "defines.h"
 
 extern int ApcupsdMain(int argc, char **argv);
@@ -88,6 +91,11 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
    char *wordPtr, *tempPtr;
    int i, quote;
 
+#ifdef HAVE_MINGW
+   InitWinAPIWrapper();
+   WSA_Init();
+#endif
+
    // Save the application instance and main thread id
    hAppInstance = hInstance;
    mainthreadId = GetCurrentThreadId();
@@ -344,22 +352,29 @@ int ApcupsdAppMain(int service)
                  MB_OK);
       _exit(0);
    }
+
    // Create a thread to handle the Windows messages
 //  (void)CreateThread(NULL, 0, Main_Msg_Loop, NULL, 0, &dwThreadID);
    pthread_create(&tid, NULL, Main_Msg_Loop, (void *)0);
 
+#if 0
    /* If service do cleanups normally done in Unix scripts */
    if (service) {
       /* Zap any left over no login file and powerfail file */
       if (unlink(NOLOGIN) != 0 && errno != ENOENT) {
-         syslog(0, "Could not unlink %s. ERR=%s\n", NOLOGIN, strerror(errno));
+         syslog(0, "Could not unlink %s: ERR=%s\n", NOLOGIN, strerror(errno));
       }
       if (unlink(PWRFAIL) != 0 && errno != ENOENT) {
-         syslog(0, "Could not unlink %s. ERR=%s\n", PWRFAIL, strerror(errno));
+         syslog(0, "Could not unlink %s: ERR=%s\n", PWRFAIL, strerror(errno));
       }
    }
+#endif
+
    // Call the "real" apcupsd
    ApcupsdMain(num_command_args, command_args);
    PostQuitMessage(0);
+#ifdef HAVE_MINGW
+   WSACleanup();
+#endif
    _exit(0);
 }
